diff --git a/src/backend/commands/copy.c b/src/backend/commands/copy.c
index bace390470..254be28ae4 100644
--- a/src/backend/commands/copy.c
+++ b/src/backend/commands/copy.c
@@ -23,6 +23,7 @@
 #include "access/sysattr.h"
 #include "access/xact.h"
 #include "access/xlog.h"
+#include "catalog/dependency.h"
 #include "catalog/pg_type.h"
 #include "commands/copy.h"
 #include "commands/defrem.h"
@@ -3067,8 +3068,19 @@ BeginCopyFrom(ParseState *pstate,
 		{
 			/* attribute is NOT to be copied from input */
 			/* use default value if one exists */
-			Expr	   *defexpr = (Expr *) build_column_default(cstate->rel,
-																attnum);
+			Expr	   *defexpr;
+
+			if (att->attidentity)
+			{
+				NextValueExpr *nve = makeNode(NextValueExpr);
+
+				nve->seqid = getOwnedSequence(RelationGetRelid(cstate->rel),
+											  attnum);
+				nve->typeId = att->atttypid;
+				defexpr = (Expr *) nve;
+			}
+			else
+				defexpr = (Expr *) build_column_default(cstate->rel, attnum);
 
 			if (defexpr != NULL)
 			{
diff --git a/src/test/regress/expected/copy2.out b/src/test/regress/expected/copy2.out
index 65e9c626b3..b4eb6e21b3 100644
--- a/src/test/regress/expected/copy2.out
+++ b/src/test/regress/expected/copy2.out
@@ -557,6 +557,21 @@ SELECT * FROM instead_of_insert_tbl;
   1 | test1
 (1 row)
 
+-- test with IDENTITY columns
+CREATE TABLE copy_tab_identity (a int GENERATED ALWAYS AS IDENTITY,
+  b text, c bigint);
+COPY copy_tab_identity FROM stdin;
+COPY copy_tab_identity (b, c) FROM stdin;
+SELECT * FROM copy_tab_identity;
+  a  |  b   |  c  
+-----+------+-----
+ 100 | foo  | 200
+ 101 | bar  | 201
+   1 | foo2 | 202
+   2 | bar2 | 203
+(4 rows)
+
+DROP TABLE copy_tab_identity;
 -- clean up
 DROP TABLE forcetest;
 DROP TABLE vistest;
diff --git a/src/test/regress/sql/copy2.sql b/src/test/regress/sql/copy2.sql
index f3a6d228fa..9bee10b7fd 100644
--- a/src/test/regress/sql/copy2.sql
+++ b/src/test/regress/sql/copy2.sql
@@ -411,6 +411,22 @@ test1
 
 SELECT * FROM instead_of_insert_tbl;
 
+-- test with IDENTITY columns
+CREATE TABLE copy_tab_identity (a int GENERATED ALWAYS AS IDENTITY,
+  b text, c bigint);
+COPY copy_tab_identity FROM stdin;
+100	foo	200
+101	bar	201
+\.
+
+COPY copy_tab_identity (b, c) FROM stdin;
+foo2	202
+bar2	203
+\.
+
+SELECT * FROM copy_tab_identity;
+
+DROP TABLE copy_tab_identity;
 
 -- clean up
 DROP TABLE forcetest;
